from pichemist.pka.pkamatcher import PKaMatcher

matcher = PKaMatcher()


def test_pka_matcher_list():
    smiles = ["CC(=O)N[C@@H](CCCN)C(C)=O", "CC(=O)N[C@@](C)(CC(=O)O)C(C)=O"]
    res = matcher.calculate_pka_from_list(smiles)
    expected = (
        [(10.4, "CC(=O)N[C@@H](CCCN)C(C)=O")],
        [(3.46, "CC(=O)N[C@@](C)(CC(=O)O)C(C)=O")],
    )
    assert res == expected, f"got {res}"


def test_pka_matcher_internal_smiles():
    fragment = "CC(=O)N[C@@H](CCCN)C(C)=O"
    res = matcher._pka_dict_from_smiles(fragment)
    expected = {"acid": [], "base": [(10.4, "CC(=O)N[C@@H](CCCN)C(C)=O")]}
    assert res == expected, f"got {res}"


def test_pka_matcher_smiles():
    fragment = "CC(=O)N[C@@H](CCCN)C(C)=O"
    res = matcher.calculate_pka_from_smiles(fragment)
    expected = {"acid": [], "base": [10.4]}
    assert res == expected, f"got {res}"



import argparse
import sys

from pichemist.api import pichemist_from_dict
from pichemist.config import PH_Q_FILE_PREFIX
from pichemist.io import generate_input
from pichemist.io import output_results
from pichemist.model import InputFormat
from pichemist.model import MODELS
from pichemist.model import OutputFormat
from pichemist.model import PKaMethod
from pichemist.utils import get_logger
from pichemist.utils import str2bool
from rdkit import RDLogger

# Configure logging
log = get_logger(__name__)
RDLogger.DisableLog("rdApp.info")

__prog__ = "pIChemiSt"
__doc__ = """Calculates the isoelectric point (pI) of a molecule by cutting
its amide bonds, retreiving the pKa values for known AAs, predicting pKas
of unknown fragments using pKaMatcher or ACDlabs, and finally calculating
the pI using the Henderson-Hasselbalch equation."""


def arg_parser(args):
    parser = argparse.ArgumentParser(prog=__prog__, description=__doc__)
    parser.add_argument(
        "-i", dest="input", help="Input filepath (or SMILES string)", default=None
    )
    parser.add_argument(
        "-if",
        dest="input_format",
        help="Format of the input",
        choices=MODELS[InputFormat],
        default=InputFormat.SMILES_FILE.value,
    )
    parser.add_argument("-o", dest="output_file", help="Output filepath", default=None)
    parser.add_argument(
        "-of",
        dest="output_format",
        help="Format of the output",
        choices=MODELS[OutputFormat],
        default=OutputFormat.CONSOLE.value,
    )
    parser.add_argument(
        "--plot_ph_q_curve",
        default=False,
        action="store_true",
        dest="plot_ph_q_curve",
        help="Generate an image of the " "pH/Q curve into a file",
    )
    parser.add_argument(
        "-pp",
        dest="ph_q_curve_file_prefix",
        help="pH/Q plot file prefix",
        default=PH_Q_FILE_PREFIX,
    )
    parser.add_argument(
        "--print_fragment_pkas",
        default=False,
        action="store_true",
        dest="print_fragment_pkas",
        help="Print the fragments with corresponding " "pKas used in pI calcution.",
    )
    parser.add_argument(
        "--generate_fragment_images",
        default=False,
        action="store_true",
        dest="generate_fragment_images",
        help="Generate 2D depiction of the frgament smiles in base64 format.",
    )
    parser.add_argument(
        "--ionizable_nterm",
        type=str2bool,
        default=True,
        dest="ionizable_nterm",
        help="Applies to FASTA input only. "
        "If set to 'false' the N-terminus is capped. "
        "If set to 'true' the N-terminus is free amine. ",
    )
    parser.add_argument(
        "--ionizable_cterm",
        type=str2bool,
        default=True,
        dest="ionizable_cterm",
        help="Applies to FASTA input only. "
        "If set to 'false' the C-terminus is capped. "
        "If set to 'true' the C-terminus is free amine. ",
    )
    parser.add_argument(
        "--method",
        choices=MODELS[PKaMethod],
        default=PKaMethod.PKA_MATCHER.value,
        help="Method for the prediction of the pKa of unknown fragments",
    )
    if not args:
        args = ["-h"]
    return parser.parse_args(args)


def run_pichemist(args):
    """High-level wrapper of pIChemiSt."""
    input_dict = generate_input(args.input_format, args.input)
    output_dict = pichemist_from_dict(
        input_dict,
        args.method,
        ph_q_curve_file_prefix=args.ph_q_curve_file_prefix,
        plot_ph_q_curve=args.plot_ph_q_curve,
        print_fragments=args.print_fragment_pkas,
        ionizable_nterm=args.ionizable_nterm,
        ionizable_cterm=args.ionizable_cterm,
        generate_fragment_images=args.generate_fragment_images,
    )
    output_results(
        input_dict,
        output_dict,
        args.output_file,
        args.output_format,
        args.method,
        args.print_fragment_pkas,
    )


def main():
    """Entry point for the CLI."""
    args = arg_parser(sys.argv[1:])
    run_pichemist(args)


if __name__ == "__main__":
    main()



import json

from pichemist.config import SS_SMARTS_PKA_SET_FILEPATH


def create_logic_set_from_standardised_json(filepath):
    """
    Reads a pKa set from standardised JSON and converts it
    into an set compatible with the legacy logic
    of the software.

    """
    with open(filepath) as f:
        data = json.load(f)

    logic_set = list()
    for smarts, v in data["smarts"].items():
        res_data = list()
        name = v["name"]
        data = v["data"]
        for d in data:
            res_data.append({
                "pka": d["pka"],
                "idx": d["idx"],
                "pka_std": d["pka_std"],
                "type": d["type"],
                "smarts": smarts,
                "name": name
            })
        logic_set.append(res_data)
    return logic_set


# Read in the data
SS_SMARTS_PKA_SET = create_logic_set_from_standardised_json(
    SS_SMARTS_PKA_SET_FILEPATH)



import argparse
import logging
import os


def get_logger(name):
    """
    Creates a logger where level is
    determined by the env LOGGING_LEVEL.

    """
    log = logging.getLogger(name)
    if os.environ.get("LOGGING_LEVEL") == "DEBUG":
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)
    return log


def str2bool(v):
    """
    Converts a string to a boolean.
    """
    if isinstance(v, bool):
        return v
    if v.lower() in ("yes", "true", "t", "y", "1"):
        return True
    elif v.lower() in ("no", "false", "f", "n", "0"):
        return False
    else:
        raise argparse.ArgumentTypeError("Boolean value expected.")



import tempfile

import pytest
from helpers import examples_dir
from helpers import jsonify_output
from pichemist.api import pichemist_from_dict
from pichemist.io import generate_input


def test_pka_matcher_json_1_file():
    """Example with mixed amino acids using pKaMatcher."""
    args = {
        "input_data": f"{examples_dir}/payload_1.smi",
        "input_format": "smiles_file",
        "plot_ph_q_curve": False,
        "print_fragments": False,
        "method": "pkamatcher",
    }

    expected = {
        "1": {
            "mol_name": "Phe-Ornithine-aMeAsp-Lys-dAla",
            "pI": {
                "IPC2_peptide": 8.046875,
                "IPC_peptide": 9.8125,
                "ProMoST": 8.375,
                "Gauci": 8.6875,
                "Grimsley": 8.9375,
                "Thurlkill": 9.0625,
                "Lehninger": 9.859375,
                "Toseland": 9.40625,
                "pI mean": 9.0234,
                "std": 1.7216,
                "err": 0.6087,
            },
            "QpH7": {
                "IPC2_peptide": 0.6315,
                "IPC_peptide": 0.9916,
                "ProMoST": 0.2617,
                "Gauci": 0.5541,
                "Grimsley": 0.6645,
                "Thurlkill": 0.7975,
                "Lehninger": 0.9932,
                "Toseland": 0.9516,
                "Q at pH7.4 mean": 0.7307,
                "std": 0.675,
                "err": 0.2386,
            },
            "pI_interval": [8.625, 9.3625],
            "pI_interval_threshold": 0.2,
            "pKa_set": "IPC2_peptide",
        }
    }

    input_dict = generate_input(args["input_format"], args["input_data"])
    output = pichemist_from_dict(
        input_dict, args["method"], args["plot_ph_q_curve"], args["print_fragments"]
    )
    assert expected == jsonify_output(output)


def test_pka_matcher_json_1_stdin():
    """Example with mixed amino acids using pKaMatcher."""
    args = {
        "input_data": "C[C@@H](NC(=O)[C@H](CCCCN)NC(=O)[C@](C)(CC(=O)O)NC(=O)[C@H](CCCN)NC(=O)[C@@H](N)Cc1ccccc1)C(=O)O",  # noqa
        "input_format": "smiles_stdin",
        "plot_ph_q_curve": False,
        "print_fragments": False,
        "method": "pkamatcher",
    }

    expected = {
        "1": {
            "mol_name": "C[C@@H](NC(=O)[C@H](CCCCN)NC(=O)[C@](C)(CC(=O)O)NC(=O)[C@H](CCCN)NC(=O)[C@@H](N)Cc1ccccc1)C(=O)O",  # noqa
            "pI": {
                "IPC2_peptide": 8.046875,
                "IPC_peptide": 9.8125,
                "ProMoST": 8.375,
                "Gauci": 8.6875,
                "Grimsley": 8.9375,
                "Thurlkill": 9.0625,
                "Lehninger": 9.859375,
                "Toseland": 9.40625,
                "pI mean": 9.0234,
                "std": 1.7216,
                "err": 0.6087,
            },
            "QpH7": {
                "IPC2_peptide": 0.6315,
                "IPC_peptide": 0.9916,
                "ProMoST": 0.2617,
                "Gauci": 0.5541,
                "Grimsley": 0.6645,
                "Thurlkill": 0.7975,
                "Lehninger": 0.9932,
                "Toseland": 0.9516,
                "Q at pH7.4 mean": 0.7307,
                "std": 0.675,
                "err": 0.2386,
            },
            "pI_interval": [8.625, 9.3625],
            "pI_interval_threshold": 0.2,
            "pKa_set": "IPC2_peptide",
        }
    }

    input_dict = generate_input(args["input_format"], args["input_data"])
    output = pichemist_from_dict(
        input_dict, args["method"], args["plot_ph_q_curve"], args["print_fragments"]
    )
    assert expected == jsonify_output(output)


def test_natural_aa_json_1():
    """Example with only natural amino acids (only FASTA matching)."""
    args = {
        "input_data": f"{examples_dir}/payload_2.smi",
        "input_format": "smiles_file",
        "plot_ph_q_curve": False,
        "print_fragments": False,
        "method": "pkamatcher",
    }

    expected = {
        "1": {
            "mol_name": "Cys-Asn-Cys-Asn",
            "pI": {
                "IPC2_peptide": 5.0,
                "IPC_peptide": 5.0,
                "ProMoST": 5.5,
                "Gauci": 5.0,
                "Grimsley": 4.875,
                "Thurlkill": 5.5,
                "Lehninger": 5.0,
                "Toseland": 4.875,
                "pI mean": 5.0938,
                "std": 0.6789,
                "err": 0.24,
            },
            "QpH7": {
                "IPC2_peptide": -0.2391,
                "IPC_peptide": -0.2318,
                "ProMoST": -0.4771,
                "Gauci": -0.937,
                "Grimsley": -1.9323,
                "Thurlkill": -0.3328,
                "Lehninger": -0.2154,
                "Toseland": -1.5909,
                "Q at pH7.4 mean": -0.7446,
                "std": 1.7898,
                "err": 0.6328,
            },
            "pI_interval": [3.7625, 6.6875],
            "pI_interval_threshold": 0.2,
            "pKa_set": "IPC2_peptide",
        }
    }

    input_dict = generate_input(args["input_format"], args["input_data"])
    output = pichemist_from_dict(
        input_dict, args["method"], args["plot_ph_q_curve"], args["print_fragments"]
    )
    assert expected == jsonify_output(output)


@pytest.mark.acd
def test_acd_json_1():
    """Example with mixed amino acids using ACD."""
    args = {
        "input_data": f"{examples_dir}/payload_1.smi",
        "input_format": "smiles_file",
        "plot_ph_q_curve": False,
        "print_fragments": False,
        "method": "acd",
    }

    expected = {
        "1": {
            "mol_name": "Phe-Ornithine-aMeAsp-Lys-dAla",
            "pI": {
                "IPC2_peptide": 8.046875,
                "IPC_peptide": 9.75,
                "ProMoST": 8.3125,
                "Gauci": 8.625,
                "Grimsley": 8.875,
                "Thurlkill": 9.0,
                "Lehninger": 9.796875,
                "Toseland": 9.34375,
                "pI mean": 8.9688,
                "std": 1.6868,
                "err": 0.5964,
            },
            "QpH7": {
                "IPC2_peptide": 0.6313,
                "IPC_peptide": 0.9914,
                "ProMoST": 0.2616,
                "Gauci": 0.5539,
                "Grimsley": 0.6644,
                "Thurlkill": 0.7974,
                "Lehninger": 0.9931,
                "Toseland": 0.9514,
                "Q at pH7.4 mean": 0.7306,
                "std": 0.675,
                "err": 0.2386,
            },
            "pI_interval": [8.6125, 9.3125],
            "pI_interval_threshold": 0.2,
            "pKa_set": "IPC2_peptide",
        }
    }

    input_dict = generate_input(args["input_format"], args["input_data"])
    output = pichemist_from_dict(
        input_dict, args["method"], args["plot_ph_q_curve"], args["print_fragments"]
    )
    assert expected == jsonify_output(output)


@pytest.mark.acd
def test_acd_json_2():
    """Example with mixed amino acids using ACD (GALAS)."""
    args = {
        "input_data": f"{examples_dir}/payload_6.smi",
        "input_format": "smiles_file",
        "plot_ph_q_curve": False,
        "print_fragments": False,
        "method": "acd",
    }

    expected = {
        "1": {
            "mol_name": "Galas789",
            "pI": {
                "IPC2_peptide": 7.875,
                "IPC_peptide": 7.75,
                "ProMoST": 8.125,
                "Gauci": 7.75,
                "Grimsley": 8.0,
                "Thurlkill": 8.0,
                "Lehninger": 7.75,
                "Toseland": 7.875,
                "pI mean": 7.8906,
                "std": 0.3724,
                "err": 0.1317,
            },
            "QpH7": {
                "IPC2_peptide": 0.0887,
                "IPC_peptide": 0.03,
                "ProMoST": 0.2262,
                "Gauci": 0.0269,
                "Grimsley": 0.127,
                "Thurlkill": 0.1116,
                "Lehninger": 0.0284,
                "Toseland": 0.0686,
                "Q at pH7.4 mean": 0.0884,
                "std": 0.1794,
                "err": 0.0634,
            },
            "pI_interval": [6.9875, 8.8],
            "pI_interval_threshold": 0.2,
            "pKa_set": "IPC2_peptide",
        }
    }

    input_dict = generate_input(args["input_format"], args["input_data"])
    output = pichemist_from_dict(
        input_dict, args["method"], args["plot_ph_q_curve"], args["print_fragments"]
    )
    assert expected == jsonify_output(output)


@pytest.mark.acd
def test_natural_aa_json_2():
    """Example with only natural amino acids (only FASTA matching)."""
    args = {
        "input_data": f"{examples_dir}/payload_2.smi",
        "input_format": "smiles_file",
        "plot_ph_q_curve": False,
        "print_fragments": False,
        "method": "acd",
    }

    expected = {
        "1": {
            "mol_name": "Cys-Asn-Cys-Asn",
            "pI": {
                "IPC2_peptide": 5.0,
                "IPC_peptide": 5.0,
                "ProMoST": 5.5,
                "Gauci": 5.0,
                "Grimsley": 4.875,
                "Thurlkill": 5.5,
                "Lehninger": 5.0,
                "Toseland": 4.875,
                "pI mean": 5.0938,
                "std": 0.6789,
                "err": 0.24,
            },
            "QpH7": {
                "IPC2_peptide": -0.2391,
                "IPC_peptide": -0.2318,
                "ProMoST": -0.4771,
                "Gauci": -0.937,
                "Grimsley": -1.9323,
                "Thurlkill": -0.3328,
                "Lehninger": -0.2154,
                "Toseland": -1.5909,
                "Q at pH7.4 mean": -0.7446,
                "std": 1.7898,
                "err": 0.6328,
            },
            "pI_interval": [3.7625, 6.6875],
            "pI_interval_threshold": 0.2,
            "pKa_set": "IPC2_peptide",
        }
    }

    input_dict = generate_input(args["input_format"], args["input_data"])
    output = pichemist_from_dict(
        input_dict, args["method"], args["plot_ph_q_curve"], args["print_fragments"]
    )
    assert expected == jsonify_output(output)


def test_natural_aa_json_3():
    """
    Example with only natural amino acids (only FASTA matching).
    It also checks for the presence of "plot_filename" in the output.

    """
    tmp_file_prefix = tempfile.NamedTemporaryFile().name
    tmp_filepath = f"{tmp_file_prefix}_1.png"
    args = {
        "input_data": f"{examples_dir}/payload_2.smi",
        "input_format": "smiles_file",
        "ph_q_curve_file_prefix": tmp_file_prefix,
        "plot_ph_q_curve": True,
        "print_fragments": False,
        "method": "pkamatcher",
    }

    expected = {
        "1": {
            "mol_name": "Cys-Asn-Cys-Asn",
            "pI": {
                "IPC2_peptide": 5.0,
                "IPC_peptide": 5.0,
                "ProMoST": 5.5,
                "Gauci": 5.0,
                "Grimsley": 4.875,
                "Thurlkill": 5.5,
                "Lehninger": 5.0,
                "Toseland": 4.875,
                "pI mean": 5.0938,
                "std": 0.6789,
                "err": 0.24,
            },
            "QpH7": {
                "IPC2_peptide": -0.2391,
                "IPC_peptide": -0.2318,
                "ProMoST": -0.4771,
                "Gauci": -0.937,
                "Grimsley": -1.9323,
                "Thurlkill": -0.3328,
                "Lehninger": -0.2154,
                "Toseland": -1.5909,
                "Q at pH7.4 mean": -0.7446,
                "std": 1.7898,
                "err": 0.6328,
            },
            "pI_interval": [3.7625, 6.6875],
            "pI_interval_threshold": 0.2,
            "plot_filename": tmp_filepath,
            "pKa_set": "IPC2_peptide",
        }
    }

    input_dict = generate_input(args["input_format"], args["input_data"])
    output = pichemist_from_dict(
        input_dict,
        args["method"],
        args["ph_q_curve_file_prefix"],
        args["plot_ph_q_curve"],
        args["print_fragments"],
    )
    assert expected == jsonify_output(output)



import json

from pichemist.config import AA_SMARTS_SET_FILEPATH


class AASmarts(object):
    """Dynamic AA SMARTS object."""
    def __init__(self, data):
        for k, v in data.items():
            setattr(self, k, v)


def create_global_object_from_standardised_json(filepath):
    """
    Reads a SMARTS set from standardised JSON and converts it
    into an object with attributes compatible with the legacy logic
    of the software.

    """
    with open(filepath) as f:
        data = json.load(f)
    return AASmarts(data["smarts"])


AA_SMARTS_SET = create_global_object_from_standardised_json(
    AA_SMARTS_SET_FILEPATH).__dict__



from pichemist.charges import SmartsChargeCalculator
from pichemist.config import REFERENCE_PKA_SET
from pichemist.core import calculate_isoelectric_interval_and_threshold
from pichemist.core import calculate_pI_pH_and_charge_dicts
from pichemist.core import compile_frags_pkas_for_output
from pichemist.core import merge_matched_and_calculated_pkas
from pichemist.fasta.matcher import FastaPKaMatcher
from pichemist.model import InputAttribute
from pichemist.model import MODELS
from pichemist.model import OutputAttribute
from pichemist.model import PKaMethod
from pichemist.molecule import MolStandardiser
from pichemist.molecule import PeptideCutter
from pichemist.pka.acd import ACDPKaCalculator
from pichemist.pka.pkamatcher import PKaMatcher
from pichemist.plot import output_ph_q_curve


class ApiException(Exception):
    pass


def fasta_pkas_from_list(smiles_list):
    """Match pKas from FASTA definitions for a SMILES list."""
    return FastaPKaMatcher().get_aa_pkas_from_list(smiles_list)


def fasta_pkas_from_aa_list(aa_list, ionizable_nterm, ionizable_cterm):
    """Match pKas from FASTA definitions for a single-letter amino acids list."""
    return FastaPKaMatcher().get_aa_pkas_from_aa_list(
        aa_list, ionizable_nterm, ionizable_cterm
    )


def calculated_pkas_from_list(smiles_list, method):
    """Calculates pKa values using ACD or pKa Matcher."""
    if method not in MODELS[PKaMethod]:
        raise NotImplementedError(
            "Invalid method. Only the formats " f"{MODELS[PKaMethod]} are accepted"
        )
    if method == PKaMethod.ACD.value:
        base_pkas, acid_pkas = ACDPKaCalculator().calculate_pka_from_list(smiles_list)
    if method == PKaMethod.PKA_MATCHER.value:
        base_pkas, acid_pkas = PKaMatcher().calculate_pka_from_list(smiles_list)
    return base_pkas, acid_pkas


def pkas_and_charges_from_aa_list(aa_list, ionizable_nterm, ionizable_cterm):
    """
    Produces the pKa values for a list of AA in single letter FASTA format

    """
    (
        base_pkas_fasta,
        acid_pkas_fasta,
        diacid_pkas_fasta,
    ) = fasta_pkas_from_aa_list(aa_list, ionizable_nterm, ionizable_cterm)

    # sic - Keeping these empty dict in output for consistency
    # TODO: Diacid dictionaries are not used, they are
    # deprecated and should be removed from the code
    base_pkas_calc = dict()
    acid_pkas_calc = dict()
    diacid_pkas_calc = dict()
    net_qs_and_frags = dict()

    return (
        base_pkas_fasta,
        acid_pkas_fasta,
        diacid_pkas_fasta,
        base_pkas_calc,
        acid_pkas_calc,
        diacid_pkas_calc,
        net_qs_and_frags,
    )


def pkas_and_charges_from_list(smiles_list, method):
    """
    Produces the pKa values for a list of SMILES by matching them against
    some FASTA definitions and calculating the unmatched ones.

    """
    # FASTA match
    (
        unknown_frags,
        base_pkas_fasta,
        acid_pkas_fasta,
        diacid_pkas_fasta,
    ) = fasta_pkas_from_list(smiles_list)

    # Unknown fragment calculation
    base_pkas_calc, acid_pkas_calc = calculated_pkas_from_list(
        unknown_frags, method=method
    )

    # Calculate charges and their matching fragments, and return results
    net_qs_and_frags = SmartsChargeCalculator().calculate_net_qs_from_list(
        unknown_frags
    )
    return (
        base_pkas_fasta,
        acid_pkas_fasta,
        diacid_pkas_fasta,
        base_pkas_calc,
        acid_pkas_calc,
        net_qs_and_frags,
    )


def pichemist_from_dict(
    input_dict,
    method,
    ph_q_curve_file_prefix=None,
    plot_ph_q_curve=False,
    print_fragments=False,
    ionizable_nterm=True,
    ionizable_cterm=True,
    generate_fragment_images=False,
):

    """Runs the full logic for a given input dictionary."""
    dict_output = dict()
    for mol_idx in input_dict.keys():
        # Prepare molecule and break into fragments
        dct = input_dict[mol_idx]
        mol_name = dct[InputAttribute.MOL_NAME.value]
        fasta = dct.get(InputAttribute.MOL_FASTA.value)
        mol = dct.get(InputAttribute.MOL_OBJECT.value)

        # Check input validity
        if not any([mol, fasta]):
            raise ApiException(
                "Invalid input format. Neither a structure not a FASTA were provided."
            )

        # MOL overrides FASTA
        if mol:
            mol = MolStandardiser().standardise_molecule(mol)
            smiles_list = PeptideCutter().break_amide_bonds_and_cap(mol)
            (
                base_pkas_fasta,
                acid_pkas_fasta,
                diacid_pkas_fasta,
                base_pkas_calc,
                acid_pkas_calc,
                net_qs_and_frags,
            ) = pkas_and_charges_from_list(smiles_list, method)

        elif fasta:
            aa_list = [char for char in fasta]
            (
                base_pkas_fasta,
                acid_pkas_fasta,
                diacid_pkas_fasta,
                base_pkas_calc,
                acid_pkas_calc,
                diacid_pkas_calc,
                net_qs_and_frags,
            ) = pkas_and_charges_from_aa_list(aa_list, ionizable_nterm, ionizable_cterm)

        # Merge pKas
        (base_pkas_dict, acid_pkas_dict,) = merge_matched_and_calculated_pkas(
            base_pkas_fasta, base_pkas_calc, acid_pkas_fasta, acid_pkas_calc
        )

        # Recomple fragments and pKa for table output
        (
            frag_acid_pkas_fasta,
            frag_base_pkas_fasta,
            frag_acid_pkas_calc,
            frag_base_pkas_calc,
            frag_Qs_calc,
        ) = compile_frags_pkas_for_output(
            base_pkas_fasta,
            acid_pkas_fasta,
            base_pkas_calc,
            acid_pkas_calc,
            net_qs_and_frags,
            generate_fragment_images=generate_fragment_images,
        )

        # Calculate the curves
        pI_dict, q_dict, pH_q_dict = calculate_pI_pH_and_charge_dicts(
            base_pkas_dict, acid_pkas_dict, net_qs_and_frags
        )

        # Calculate isoelectric interval
        interval, interval_threshold = calculate_isoelectric_interval_and_threshold(
            pH_q_dict
        )

        # Output for given molecule
        dict_output[mol_idx] = {
            OutputAttribute.MOL_NAME.value: mol_name,
            OutputAttribute.PI.value: pI_dict,
            OutputAttribute.Q_PH7.value: q_dict,
            OutputAttribute.PI_INTERVAL.value: interval,
            OutputAttribute.PI_INTERVAL_THRESHOLD.value: interval_threshold,
        }

        # Plot pH/Q curve
        if plot_ph_q_curve and not isinstance(ph_q_curve_file_prefix, str):
            raise ApiException(
                "A file prefix for the pH/Q curve plots must " "be specified."
            )
        if plot_ph_q_curve:
            fig_filename = f"{ph_q_curve_file_prefix}_{mol_idx}.png"
            output_ph_q_curve(pH_q_dict, fig_filename)
            dict_output[mol_idx][OutputAttribute.PLOT_FILENAME.value] = fig_filename

        # Define set for reporting pKa of individual amino acids and fragments
        dict_output[mol_idx].update({OutputAttribute.PKA_SET.value: REFERENCE_PKA_SET})

        if print_fragments:
            dict_output[mol_idx].update(
                {
                    OutputAttribute.FRAG_BASE_PKA_FASTA.value: frag_base_pkas_fasta,
                    OutputAttribute.FRAG_ACID_PKA_FASTA.value: frag_acid_pkas_fasta,
                    OutputAttribute.FRAG_BASE_PKA_CALC.value: frag_base_pkas_calc,
                    OutputAttribute.FRAG_ACID_PKA_CALC.value: frag_acid_pkas_calc,
                    OutputAttribute.FRAG_CONSTANT_QS.value: frag_Qs_calc,
                }
            )
    return dict_output



from pichemist.fasta.scrambler import FastaScrambler

scrambler = FastaScrambler()


def test_fasta_scrambler_list():
    """Produces a FASTA sequence from a set of amino acids."""
    smiles = [
        "CC(=O)N[C@@H](C)C(C)=O",
        "CC(=O)N[C@@H](CCC(=O)O)C(=O)O",
        "CC(=O)N[C@@H](Cc1ccc(O)cc1)C(C)=O",
        "CC(=O)N[C@H](C(C)=O)C(C)C",
        "CC(=O)[C@@H](N)Cc1ccccc1",
        "CC(=O)[C@@H]1CCCN1C(C)=O"
    ]
    res = scrambler.get_scrambled_fasta_from_list(smiles)
    expected = "AEYVFP"
    assert res == expected, f"got {res}"



import os

from legacy import promost
from legacy_conversion import convert_legacy_set_to_json


def build_promost_set_from_module(pm: promost):
    """Builds a pKa set from a module."""
    pka_set = {
        "acidic": pm.acidic,
        "basic": pm.basic,
        "terminus_ionizable": pm.terminus_ionizable
    }
    return pka_set


if __name__ == '__main__':
    # Get parent dir where the script runs
    parent_dir = os.path.dirname(os.path.realpath(__file__))

    # Converts the legacy data set with filled missing AAs into JSON
    name = promost.SetName
    pka_set = build_promost_set_from_module(promost)
    filepath = f"{parent_dir}/standardised/" \
        f"pka_set_{name.lower()}_standardised.json"
    pka_dict = convert_legacy_set_to_json(pka_set, name, filepath)



from pichemist.charges import SmartsChargeCalculator

qcalc = SmartsChargeCalculator()


def test_charges_smiles_1():
    res = qcalc.calculate_net_qs_from_smiles("CCCC[N+](C)(C)CC[N+](C)(C)C")
    expected = 2
    assert res == expected, f"got {res}"


def test_charges_smiles_2():
    res = qcalc.calculate_net_qs_from_smiles("CCCC[N+](C)(C)CC[O-]")
    expected = 1
    assert res == expected, f"got {res}"


def test_charges_smiles_3():
    res = qcalc.calculate_net_qs_from_smiles("CCCC[N+](C)(C)C[B-]")
    expected = 1
    assert res == expected, f"got {res}"


def test_charges_smiles_4():
    res = qcalc.calculate_net_qs_from_list(["CCCC[N+](C)(C)[N+]C"])
    expected = [(1, 'CCCC[N+](C)(C)[N+]C'), (1, 'CCCC[N+](C)(C)[N+]C')]
    assert res == expected, f"got {res}"



import os
import json

from pichemist.config import FASTA_PKA_SETS_DIR


def create_logic_set_from_standardised_json(filepath):
    """
    Reads a pKa set from standardised JSON and converts it
    into a set that is compatible with the legacy logic
    of the software. It returns the set and its name.

    A standardise JSON follows the format:
    {
        "name": $name-of-the-set
        "values": {
            "acidic": {
                $amino-acid: [
                    {"value": $value,
                     "position": $position},
                    {"value": $value,
                     "position": $position}
                ]
            },
            "basic": {...},
            "terminus_ionizable": {...}
        }
    }

    """
    with open(filepath) as f:
        data = json.load(f)

    logic_set = dict()
    values = data["values"]
    for value_type in values.keys():

        # Initialise a dict and populate it using
        # a list of values instead of objects
        logic_set[value_type] = dict()
        pka_type_set = values[value_type]
        for aa, values_list in pka_type_set.items():
            value_list = list()
            for val_obj in values_list:
                pka = val_obj["value"]
                value_list.append(pka)
            logic_set[value_type][aa] = value_list
    return logic_set, data["name"]


def _generate_pka_sets():
    """
    Produces a set of pKa sets that are compatible
    with the business logic.

    """
    # Get all the pKa file paths
    pka_sets = dict()
    standardised_pka_sets_dir = FASTA_PKA_SETS_DIR
    with os.scandir(standardised_pka_sets_dir) as pka_files:
        pka_filepaths = [f"{standardised_pka_sets_dir}/{pka_file.name}"
                         for pka_file in pka_files]

        for filepath in pka_filepaths:
            data_set, name = create_logic_set_from_standardised_json(filepath)
            pka_sets[name] = data_set
    return pka_sets


FASTA_PKA_SETS = _generate_pka_sets()



import pichemist
from pichemist.model import ACDPKaFlag

# Shared config
ROUNDING_DIGITS = 4

# FASTA definitions
KNOWN_BASIC_RESIDUES = ["K", "R", "H"]
KNOWN_ACIDIC_RESIDUES = ["D", "E", "C", "Y", "U"]
KNOWN_RESIDUES = [
    "G",
    "A",
    "S",
    "P",
    "V",
    "T",
    "C",
    "I",
    "L",
    "N",
    "D",
    "Q",
    "K",
    "E",
    "M",
    "H",
    "F",
    "R",
    "Y",
    "W",
    "X",
    "Z",
    "B",
    "U",
]

PKA_JSON_INDICES = {"primary": 0, "n-terminal": 1, "c-terminal": 2}

REFERENCE_PKA_SET = "IPC2_peptide"

PKA_SETS_NAMES = [
    "IPC2_peptide",
    "IPC_peptide",
    "ProMoST",
    "Gauci",
    "Grimsley",
    "Thurlkill",
    "Lehninger",
    "Toseland",
]

ALL_KNOWN_PKA_SETS_NAMES = [
    "ProMoST",
    "IPC_peptide",
    "IPC2_peptide",
    "Gauci",
    "Bjellqvist",
    "Rodwell",
    "Grimsley",
    "Thurlkill",
    "EMBOSS",
    "DTASelect",
    "Solomon",
    "Sillero",
    "Lehninger",
    "Toseland",
    "Nozaki",
    "Dawson",
]

# SMARTS definitions
SKIP_SMARTS_NAMES = []
PKA_LIMITS = {
    "acid_1": -5,
    "acid_2": 12,
    "base_1": 2,
    "base_2": 15,
}
AA_TYPE_KEYS = {
    "capped": "d_capped_aa_smarts",
    "nterm": "d_nterm_free_aa_smarts",
    "cterm": "d_cterm_free_aa_smarts",
}

# ACD configuration
ACD_METHOD = ACDPKaFlag.GALAS

# Plot definitions
PLOT_LINE_WIDTHS = {"w1": 4.0, "w2": 3.0, "w3": 2.0, "w4": 1.0}

# Paths
MODULE_DIR = pichemist.__path__[0]
DATA_DIR = f"{MODULE_DIR}/data"
FASTA_PKA_DATA_DIR = f"{DATA_DIR}/fasta"
FASTA_PKA_SETS_DIR = f"{FASTA_PKA_DATA_DIR}/standardised"
SMARTS_PKA_DATA_DIR = f"{DATA_DIR}/smarts"
SS_SMARTS_PKA_SET_FILEPATH = (
    f"{SMARTS_PKA_DATA_DIR}/standardised/ss_smarts_set_standardised.json"
)
AA_SMARTS_SET_FILEPATH = (
    f"{SMARTS_PKA_DATA_DIR}/standardised/aa_smarts_set_standardised.json"
)
PH_Q_FILE_PREFIX = "out_ph_q_curve"



import os
import json

import legacy.smarts_naturalAAs


def _get_aa_dict_names():
    return [i for i in legacy.smarts_naturalAAs.__dir__()
            if not i.startswith("__")]


def convert_legacy_aa_smarts_to_json(output_filepath):
    """Conversion of AA SMARTS into JSON."""
    result = {"smarts": {}}
    for d in _get_aa_dict_names():
        result["smarts"][d] = getattr(legacy.smarts_naturalAAs, d)
    with open(output_filepath, "w") as f:
        json.dump(result, f, indent=2)


if __name__ == '__main__':
    # Get parent dir where the script runs
    parent_dir = os.path.dirname(os.path.realpath(__file__))

    # Converts the legacy data set into JSON
    filepath = f"{parent_dir}/standardised/" \
        f"aa_smarts_set_standardised.json"
    convert_legacy_aa_smarts_to_json(filepath)



from pichemist.config import PKA_LIMITS
from pichemist.config import SKIP_SMARTS_NAMES
from pichemist.model import MODELS
from pichemist.model import PKaType
from pichemist.smarts.pka_set import SS_SMARTS_PKA_SET
from rdkit import Chem


class PKaMatcher(object):
    """
    Uses a set of SMARTS definitions and their pKa annotations
    against input SMILES to match their pKa(s).

    """

    def __init__(self):
        self.smarts_set = SS_SMARTS_PKA_SET
        self.skip_names = SKIP_SMARTS_NAMES

    def _pka_dict_from_smiles(self, smiles: str):
        """
        Iterates through a list of dicts
        where each dict represents a group with
        its SMARTS and pKa value. If the SMARTS matches
        the input SMILES argument, then its value, and
        the SMILES are added to the results.

        """
        results = {PKaType.ACIDIC.value: list(), PKaType.BASIC.value: list()}
        mol = Chem.MolFromSmiles(smiles)
        all_used_idxs_set = set()
        # Each group is described as a list of dicts
        for pka_group_list in self.smarts_set:
            for pka_dict in pka_group_list:

                # Skip if name is included in skip config
                if pka_dict["name"] in self.skip_names:
                    continue

                # Match SMARTS against the input and store pKa
                used_idx_list = list()  # Used within larger scope
                used_idx_local = list()  # Used within smaller scope
                pat = Chem.MolFromSmarts(pka_dict["smarts"])
                for match in mol.GetSubstructMatches(pat):
                    match_idx = match[pka_dict["idx"] - 1]
                    used_idx_list += match
                    match_idxs = set(match)
                    available_idxs = match_idxs.difference(all_used_idxs_set)
                    pka = pka_dict["pka"]
                    if pka_dict["type"] not in MODELS[PKaType]:
                        raise PKaMatcherException(
                            f"Unknown site type (got {pka_dict['type']})"
                        )

                    # Basic
                    if pka_dict["type"] == PKaType.BASIC.value:
                        if (
                            match_idx in available_idxs
                            and match_idx not in used_idx_local
                        ):
                            if (
                                pka > PKA_LIMITS["base_1"]
                                and pka < PKA_LIMITS["base_2"]
                            ):
                                results["base"].append((pka, smiles))
                    # Acidic
                    if pka_dict["type"] == PKaType.ACIDIC.value:
                        if (
                            match_idx in available_idxs
                            and match_idx not in used_idx_local
                        ):
                            if (
                                pka > PKA_LIMITS["acid_1"]
                                and pka < PKA_LIMITS["acid_2"]
                            ):
                                results["acid"].append((pka, smiles))
                    used_idx_local.append(match_idx)
            all_used_idxs_set = all_used_idxs_set.union(set(used_idx_list))
        return results

    def calculate_pka_from_smiles(self, smiles: str):
        """Calculates the pKa values for a SMILES."""
        results = self._pka_dict_from_smiles(smiles)
        return {k: [t[0] for t in v] for k, v in results.items()}

    def calculate_pka_from_list(self, smiles_list: list):
        """Calculates the pKa values for a list of SMILES."""
        base_pkas = list()
        acid_pkas = list()

        for smiles in smiles_list:
            results = self._pka_dict_from_smiles(smiles)
            base_pkas.extend(results[PKaType.BASIC.value])
            acid_pkas.extend(results[PKaType.ACIDIC.value])
        return (base_pkas, acid_pkas)


class PKaMatcherException(Exception):
    def __init__(self):
        pass



import os
import json

from legacy import smarts_notes
from legacy.smarts_pKaMatcher import read_smarts_pKaMatcher


def _add_notes_to_smarts_dict(smarts_dict):
    smarts_dict["notes"] = smarts_notes.notes
    return smarts_dict


def convert_legacy_ss_smarts_to_json(output_filepath):
    """
    Conversion of DAT semi-structured data into JSON.
    It also adds notes on how the data set was created.

    """
    data = read_smarts_pKaMatcher()
    result = dict()
    result_smarts = dict()
    for e in data:
        res_data = list()
        for i in e:
            smarts = i["smarts"]
            name = i["name"]
            res_data.append({
                "pka": i["pka"],
                "pka_std": i["pka_std"],
                "idx": i["ind"],
                "type": i["type"]
            })

            result_smarts[smarts] = {
                "name": name,
                "data": res_data
            }
    result["smarts"] = result_smarts
    result = _add_notes_to_smarts_dict(result)
    with open(output_filepath, "w") as f:
        json.dump(result, f, indent=2)


if __name__ == '__main__':
    # Get parent dir where the script runs
    parent_dir = os.path.dirname(os.path.realpath(__file__))

    # Converts the legacy data set into JSON
    filepath = f"{parent_dir}/standardised/" \
        f"ss_smarts_set_standardised.json"
    convert_legacy_ss_smarts_to_json(filepath)



#!/usr/bin/python
# module to read "smarts_pKaMatcher.dat" into list_smarts_pka
import os

def read_smarts_pKaMatcher():
    list_smarts_pka = []
    pwd = os.path.dirname(os.path.realpath(__file__))
    substructures_smarts_file = "{}/{}".format(pwd, "smarts_pKaMatcher.dat")
    with open(substructures_smarts_file,'r') as f:
        for line in f.readlines():
            ln=line.split()
            if len(ln) == 0: continue
            if line[0] == '#': continue
            substr_name = ln[0]
            substr_smarts = ln[1]

            # Loop to read ionization data. Can have several ionizations for each smarts.
            pka_l=[]
            for i in range(int(len(ln[2:])/4)):
                
               pl = ln[2+i*4:2+i*4+4] # offset of 2 and 4 entries for each pKa value: ato index, pka, pka_std, type
               pd = {'pka': float(pl[1]), 'ind':int(pl[0]),'pka_std':float(pl[2]),'type':pl[3],'smarts':substr_smarts,'name':substr_name} 
               pka_l.append(pd)

            list_smarts_pka.append(pka_l)        
    return list_smarts_pka

# Read in the data
list_smarts_pka = read_smarts_pKaMatcher()



import csv
import json
import os

from pichemist.model import InputAttribute
from pichemist.model import InputFileExtension
from pichemist.model import InputFormat
from pichemist.model import MODELS
from pichemist.model import OutputAttribute
from pichemist.model import OutputConsoleFormat
from pichemist.model import OutputFileFormat
from rdkit import Chem


class FileFormatError(Exception):
    pass


class IOException(Exception):
    pass


def generate_input(input_format, input_data):
    """Produces an input dictionary compatible with the API."""
    input_dict = dict()
    if input_format == InputFormat.SMILES_FILE or input_format == InputFormat.SD_FILE:
        input_dict = read_structure_file(input_data)
    if input_format == InputFormat.SMILES_STDIN:
        input_dict[1] = {
            InputAttribute.MOL_NAME.value: input_data,
            InputAttribute.MOL_OBJECT.value: Chem.MolFromSmiles(input_data),
            InputAttribute.MOL_FASTA.value: None,
        }
    if input_format == InputFormat.FASTA_STDIN:
        input_dict[1] = {
            InputAttribute.MOL_NAME.value: input_data,
            InputAttribute.MOL_OBJECT.value: None,
            InputAttribute.MOL_FASTA.value: input_data,
        }
    if input_format == InputFormat.FASTA_FILE:
        input_dict = read_fasta_file(input_data)
    return input_dict


def read_structure_file(input_filepath):
    """
    Reads a file containing molecule structures.
    It guesses the input type according to the extension
    of the file.

    """
    # Initialize file reader
    ext = _get_input_extension(input_filepath)
    try:
        if ext[1:] == InputFileExtension.SDF.value:
            suppl = Chem.SDMolSupplier(input_filepath)
        elif ext[1:] == InputFileExtension.SMILES.value:
            suppl = Chem.SmilesMolSupplier(input_filepath, titleLine=False)
        else:
            raise FileFormatError(
                "Invalid format. Only the formats "
                f"{MODELS[InputFileExtension]} are accepted"
            )
    except OSError:
        raise OSError(f"File error: Does the file exist? {input_filepath}")

    # Populate input and assign properties
    dict_input = dict()
    uuid = 1
    for mol in suppl:
        if not mol.HasProp("_Name"):
            mol.SetProp("_Name", "tmp_" + str(uuid))
        dict_input[uuid] = {
            InputAttribute.MOL_NAME.value: mol.GetProp("_Name"),
            InputAttribute.MOL_OBJECT.value: mol,
            InputAttribute.MOL_FASTA.value: None,
        }
        uuid += 1
    return dict_input


def read_fasta_file(input_filepath):
    """
    Reads a file containing FASTA entries using BioPython.

    """
    from Bio import SeqIO  # conditional import

    # Initialize file reader
    ext = _get_input_extension(input_filepath)
    try:
        if ext[1:] == InputFileExtension.FASTA.value:
            suppl = SeqIO.parse(open(input_filepath), "fasta")
        else:
            raise FileFormatError(
                "Invalid format. Only the formats "
                f"{MODELS[InputFileExtension]} are accepted"
            )
    except OSError:
        raise OSError(f"File error: Does the file exist? {input_filepath}")

    # Populate input and assign properties
    dict_input = dict()
    uuid = 1
    for fasta in suppl:
        dict_input[uuid] = {
            InputAttribute.MOL_NAME.value: fasta.id,
            InputAttribute.MOL_OBJECT.value: None,
            InputAttribute.MOL_FASTA.value: str(fasta.seq),
        }
        uuid += 1
    return dict_input


def _get_input_extension(input_filepath):
    _, ext = os.path.splitext(input_filepath)
    if not ext:
        raise FileFormatError("Something is wrong with the file " f"{input_filepath}")
    return ext


def _format_results_for_console_output(prop_dict, prop):
    """Prints a formatted output for a dictionary of results."""
    lj = 12
    keys = list(prop_dict.keys())
    keys.remove("std")
    keys.insert(0, "std")
    keys.remove("err")
    keys.insert(0, "err")
    keys.remove(prop + " mean")
    keys.insert(0, prop + " mean")
    print("")
    print("=" * 150)
    print(prop)
    print("-" * 33)
    for k in keys:
        p = prop_dict[k]
        print(k.rjust(lj) + "  " + str(round(p, 2)).ljust(lj))
    print("")
    return


def _output_json_to_console(dict_output):
    """Dumps a JSON string to console."""
    print(json.dumps(dict_output, indent=2))


def _output_text_to_console(dict_output, method, print_fragments=False):
    """Generates a formatted text output to console."""
    for mol_idx in dict_output.keys():
        dict_mol = dict_output[mol_idx]
        _format_results_for_console_output(dict_mol[OutputAttribute.PI.value], "pI")
        _format_results_for_console_output(
            dict_mol[OutputAttribute.Q_PH7.value], "Q at pH7.4"
        )

        int_tr = dict_mol[OutputAttribute.PI_INTERVAL_THRESHOLD.value]

        print(
            "\npH interval with charge between %4.1f and %4.1f and "
            "prediction tool: %s" % (-int_tr, int_tr, method)
        )
        print(
            "pI interval: %4.1f - %4.1f"
            % (
                dict_mol["pI_interval"][0],
                dict_mol[OutputAttribute.PI_INTERVAL.value][1],
            )
        )

        if print_fragments:
            frag_base_pkas_fasta = dict_mol[OutputAttribute.FRAG_BASE_PKA_FASTA.value]
            frag_acid_pkas_fasta = dict_mol[OutputAttribute.FRAG_ACID_PKA_FASTA.value]
            frag_base_pkas_calc = dict_mol[OutputAttribute.FRAG_BASE_PKA_CALC.value]
            frag_acid_pkas_calc = dict_mol[OutputAttribute.FRAG_ACID_PKA_CALC.value]
            frag_constant_Qs_calc = dict_mol[OutputAttribute.FRAG_CONSTANT_QS.value]

            # Print the results
            print("\nList of calculated pKa values or constant charges")
            format_header = "{:15s} {:5s}  {:17s}  {:s}"
            format_results = "{:15s} {:5d}  {:17.1f}  {:s}"
            print(
                format_header.format("Type", "Count", "pKa_or_constant_Q", "Fragment")
            )

            for _, frag in frag_base_pkas_fasta.items():
                print(
                    format_results.format(
                        frag["type"], frag["count"], frag["pka"], frag["frag"]
                    )
                )

            for _, frag in frag_base_pkas_calc.items():
                print(
                    format_results.format(
                        frag["type"], frag["count"], frag["pka"], frag["frag"]
                    )
                )

            for _, frag in frag_acid_pkas_fasta.items():
                print(
                    format_results.format(
                        frag["type"], frag["count"], frag["pka"], frag["frag"]
                    )
                )

            for _, frag in frag_acid_pkas_calc.items():
                print(
                    format_results.format(
                        frag["type"], frag["count"], frag["pka"], frag["frag"]
                    )
                )

            for _, frag in frag_constant_Qs_calc.items():
                print(
                    format_results.format(
                        frag["type"], frag["count"], frag["pka"], frag["frag"]
                    )
                )


def output_results(
    input_dict, output_dict, output_file, output_format, method, print_fragment_pkas
):
    """Interface to dispatch different outputs."""
    if not output_file:
        _check_supported_output_console_format(output_format)
        _output_to_console(output_dict, output_format, method, print_fragment_pkas)
    else:
        _check_supported_output_file_format(output_format)
        _output_to_file(input_dict, output_dict, output_file, output_format)
        _print_summary_to_console(output_dict, output_file)


def _check_supported_output_console_format(output_format):
    """Checks the supported output format."""
    if output_format not in MODELS[OutputConsoleFormat]:
        raise IOException(
            "Only the following output formats "
            f"are supported: {MODELS[OutputConsoleFormat]}"
        )


def _output_to_console(output_dict, output_format, method, print_fragment_pkas):
    """Deals with console outputs."""
    if output_format == OutputConsoleFormat.JSON:
        _output_json_to_console(output_dict)
    elif output_format == OutputConsoleFormat.CONSOLE:
        _output_text_to_console(output_dict, method, print_fragment_pkas)


def _check_supported_output_file_format(output_format):
    """Checks the supported output format."""
    if output_format not in MODELS[OutputFileFormat]:
        raise IOException(
            "Only the following file output formats "
            f"are supported: {MODELS[OutputFileFormat]} "
            f"(got {output_format})"
        )


def _prepare_output_list(input_dict, output_dict):
    """Merges input and output to prepare a list of results."""
    mol_list = list()
    for mi in input_dict.keys():
        mol = input_dict[mi][InputAttribute.MOL_OBJECT.value]
        mol.SetProp(
            "pI mean", "%.2f" % output_dict[mi][OutputAttribute.PI.value]["pI mean"]
        )
        mol.SetProp("pI std", "%.2f" % output_dict[mi][OutputAttribute.PI.value]["std"])
        mol.SetProp(
            "pI interval lower bound",
            "%.2f" % output_dict[mi][OutputAttribute.PI_INTERVAL.value][0],
        )
        mol.SetProp(
            "pI interval upper bound",
            "%.2f" % output_dict[mi][OutputAttribute.PI_INTERVAL.value][1],
        )
        mol.SetProp(
            "pI interval threshold",
            "%.2f" % output_dict[mi][OutputAttribute.PI_INTERVAL_THRESHOLD.value],
        )
        mol_list.append(mol)
    return mol_list


def _output_csv_to_file(mol_list, output_file):
    """Dumps the results to a CSV file."""
    with open(output_file, "w") as csv_f:
        csv_w = csv.writer(csv_f)
        count = 0
        for mol in mol_list:
            props = mol.GetPropsAsDict()
            count += 1
            if count == 1:
                header = [OutputAttribute.SMILES.value] + list(props.keys())
                csv_w.writerow(header)
            row = [Chem.MolToSmiles(mol)]
            for p in header[1:]:
                row += [props[p]]
            csv_w.writerow(row)


def _output_to_file(input_dict, output_dict, output_file, output_format):
    """Deals with file outputs."""
    mol_list = _prepare_output_list(input_dict, output_dict)
    if output_format == OutputFileFormat.SD_FILE:
        with Chem.SDWriter(output_file) as sdf_w:
            for mol in mol_list:
                sdf_w.write(mol)

    elif output_format == OutputFileFormat.CSV_FILE:
        _output_csv_to_file(mol_list, output_file)


def _print_summary_to_console(output_dict, output_file):
    """Print summary to console."""
    dict_file = {
        "output_file": output_file,
        "output_info": "Number of molecules processed: " f"{len(output_dict.keys())}",
    }
    print(json.dumps(dict_file))



import os
import json

from legacy_conversion import _invert_dict
from pichemist.config import KNOWN_BASIC_RESIDUES
from pichemist.config import KNOWN_ACIDIC_RESIDUES
from pichemist.config import KNOWN_RESIDUES
from pichemist.config import PKA_JSON_TYPE_MATCHING
from pichemist.config import PKA_JSON_INDICES


def enrich_short_pka_set(pka_set):
    """
    Iterates through a short pKa set
    and enriches the data to produce
    a set that is enriched (i.e., ProMoST format).

    """
    basic = dict()
    acidic = dict()
    terminus_ionizable = dict()

    for res in KNOWN_BASIC_RESIDUES:
        if res in pka_set.keys():
            pka = pka_set[res]
            basic[res] = [pka, pka, pka]

    for res in KNOWN_ACIDIC_RESIDUES:
        if res in pka_set.keys():
            pka = pka_set[res]
            acidic[res] = [pka, pka, pka]

    for res in KNOWN_RESIDUES:
        pka_cterm = pka_set['Cterm']
        pka_nterm = pka_set['Nterm']
        terminus_ionizable[res] = [pka_nterm, pka_cterm]

    pka_set = {
     'acidic': acidic,
     'basic': basic,
     'terminus_ionizable': terminus_ionizable
    }
    return pka_set


def convert_original_set_to_standardised_json(pka_set, name,
                                              output_filepath):
    """
    Converts an original short pKa set
    into a standardised JSON.

    """
    pka_dict = dict()
    pka_dict["name"] = name
    values = {
         "acidic": dict(),
         "basic": dict(),
         "terminus_ionizable": dict()
    }
    pka_dict["values"] = values

    # NOTE: new_name and original_name are the same now
    for new_name, original_name in PKA_JSON_TYPE_MATCHING.items():
        pka_type_set = pka_set[original_name]

        # For each pKa type set (e.g. acidic)
        # Create a new set of values with value and position attributes
        for aa, value_list in pka_type_set.items():
            new_values = list()
            for i in range(len(value_list)):
                pka = value_list[i]
                position = _invert_dict(PKA_JSON_INDICES)[i]
                new_values.append({
                    "position": position,
                    "value": pka
                })
            values[new_name][aa] = new_values
    with open(output_filepath, "w") as f:
        json.dump(pka_dict, f, indent=2)


if __name__ == '__main__':
    # Get parent dir where the script runs
    parent_dir = os.path.dirname(os.path.realpath(__file__))

    # Get all the pKa file paths
    original_pka_sets_dir = f"{parent_dir}/original_short"
    with os.scandir(original_pka_sets_dir) as pka_files:
        pka_filepaths = [f"{original_pka_sets_dir}/{pka_file.name}"
                         for pka_file in pka_files]

    # Convert all original pKa files into standardised
    standardised_pka_sets_dir = f"{parent_dir}/standardised"
    for filepath in pka_filepaths:
        with open(filepath) as f:
            data = json.load(f)

        name = data["name"]
        pka_values = data["values"]
        enriched_pka_set = enrich_short_pka_set(pka_values)
        filepath = f"{standardised_pka_sets_dir}/" \
                   f"pka_set_{name.lower()}_standardised.json"
        convert_original_set_to_standardised_json(enriched_pka_set, name,
                                                  filepath)



import json

from pichemist.config import PKA_JSON_TYPE_MATCHING
from pichemist.config import PKA_JSON_INDICES


def _invert_dict(my_map):
    """Inverts keys and values of a dictionary."""
    inv_map = {v: k for k, v in my_map.items()}
    return inv_map


def convert_legacy_set_to_json(pka_set, name,
                               output_filepath):
    """
    Converts the legacy data into JSON.

    """
    pka_dict = dict()
    pka_dict["name"] = name
    values = {
            "acidic": {},
            "basic": {},
            "terminus_ionizable": {}
    }

    for new_name, original_name in PKA_JSON_TYPE_MATCHING.items():
        pka_type_set = pka_set[original_name]
        for aa, value_list in pka_type_set.items():
            new_values = list()
            for i in range(len(value_list)):
                pka = value_list[i]
                position = _invert_dict(PKA_JSON_INDICES)[i]
                new_values.append({
                    "position": position,
                    "value": pka
                })
            values[new_name][aa] = new_values

    pka_dict["values"] = values
    with open(output_filepath, "w") as f:
        json.dump(pka_dict, f, indent=2)
